@@ -1,14 +1,19 @@
 from typing import Any, List, Type
-import abc
 
 
 class Error(TypeError):
-    @abc.abstractmethod
     def __init__(self, ty: Type[Any], value: Any, path: List[str] = []):
+        if type(self) == Error:
+            raise ValueError(
+                "Internal Error: `Error` must not be instantiated directly"
+            )
         self.ty = ty
         self.value = value
         self.path = path
 
+    def __str__(self) -> str:
+        raise NotImplementedError()
+
 
 def _path_to_str(path: List[str]) -> str:
     return " -> ".join(reversed(path))
@@ -35,3 +40,13 @@ class MissingKeyError(Error):
     def __str__(self):
         path = _path_to_str(self.path)
         return f"Key Error in '{path}': Got '{self.key}' in 'self.value' but doesn't exist in '{self.ty}'"
+
+
+class UnsupportedTypeError(Error):
+    def __init__(self, ty: Type[Any], value: Any, path: List[str] = []):
+        self.ty = ty
+        self.value = value
+        self.path = path
+
+    def __str__(self) -> str:
+        return f"UnsupportedType: {self.ty}"
